/* See blowfish.test_data */

#include "blowfish.h"

#include <stdio.h>

char all_ok;

char
test_0 (unsigned long kl, unsigned long kr,
	unsigned long pl, unsigned long pr,
	unsigned long cl, unsigned long cr)
{
  struct blf_ctx context;
  unsigned char key[8];
  unsigned long dl, dr;

  key[0] = (kl>>24)&0xff;
  key[1] = (kl>>16)&0xff;
  key[2] = (kl>>8)&0xff;
  key[3] = kl&0xff;
  key[4] = (kr>>24)&0xff;
  key[5] = (kr>>16)&0xff;
  key[6] = (kr>>8)&0xff;
  key[7] = kr&0xff;
  Blowfish_initialize (&context, key, 8);
  dl = pl;  dr = pr;
  Blowfish_encipher (&context, &dl, &dr);
  if ( ! ( dl == cl && dr == cr ) )
    return 0;
  Blowfish_decipher (&context, &dl, &dr);
  if ( ! ( dl == pl && dr == pr ) )
    return 0;
  return 1;
}

void
do_test_0 (int num,
	   unsigned long kl, unsigned long kr,
	   unsigned long pl, unsigned long pr,
	   unsigned long cl, unsigned long cr)
{
  if ( test_0 (kl, kr, pl, pr, cl, cr) )
    printf ("Test0 number %d passed successfully.\n", num);
  else
    {
      printf ("Test0 number %d FAILED!\n", num);
      all_ok = 0;
    }
}

char
test_1 (int key_len,
	 unsigned long cl, unsigned long cr)
{
  struct blf_ctx context;
  const unsigned char key[24] = {
    0xf0, 0xe1, 0xd2, 0xc3, 0xb4, 0xa5, 0x96, 0x87,
    0x78, 0x69, 0x5a, 0x4b, 0x3c, 0x2d, 0x1e, 0x0f,
    0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77
  };
  const unsigned long pl = 0xfedcba98;
  const unsigned long pr = 0x76543210;
  unsigned long dl, dr;
  
  Blowfish_initialize (&context, key, key_len);
  dl = pl;  dr = pr;
  Blowfish_encipher (&context, &dl, &dr);
  if ( ! ( dl == cl && dr == cr ) )
    return 0;
  Blowfish_decipher (&context, &dl, &dr);
  if ( ! ( dl == pl && dr == pr ) )
    return 0;
  return 1;
}

void
do_test_1 (int key_len,
	    unsigned long cl, unsigned long cr)
{
  if ( test_1 (key_len, cl, cr) )
    printf ("Test1 passed successfully for key length %d.\n", key_len);
  else
    {
      printf ("Test1 FAILED for key length %d!\n", key_len);
      all_ok = 0;
    }
}

void
store (unsigned char byte, void *user_data)
{
  *((*(unsigned char **)user_data)++) = byte;
}

char
test_2 (char cfb)
{
  const unsigned char key[16] = {
    0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
    0xf0, 0xe1, 0xd2, 0xc3, 0xb4, 0xa5, 0x96, 0x87
  };
  const unsigned long ivl = 0xfedcba98;
  const unsigned long ivr = 0x76543210;
  const unsigned char data[29] = "7654321 Now is the time for ";
  unsigned char cipher[32];
  unsigned char decipher[32];
  unsigned char *ptr;
  struct blf_cbc_ctx context_cbc;
  const unsigned char expect_cbc[32] = {
    0x6b, 0x77, 0xb4, 0xd6, 0x30, 0x06, 0xde, 0xe6,
    0x05, 0xb1, 0x56, 0xe2, 0x74, 0x03, 0x97, 0x93,
    0x58, 0xde, 0xb9, 0xe7, 0x15, 0x46, 0x16, 0xd9,
    0x59, 0xf1, 0x65, 0x2b, 0xd5, 0xff, 0x92, 0xcc
  };
  struct blf_cfb_ctx context_cfb;
  const unsigned char expect_cfb[29] = {
    0xe7, 0x32, 0x14, 0xa2, 0x82, 0x21, 0x39, 0xca,
    0xf2, 0x6e, 0xcf, 0x6d, 0x2e, 0xb9, 0xe7, 0x6e,
    0x3d, 0xa3, 0xde, 0x04, 0xd1, 0x51, 0x72, 0x00,
    0x51, 0x9d, 0x57, 0xa6, 0xc3
  };
  int j;

  if ( ! cfb )
    {
      ptr = cipher;
      Blowfish_cbc_start_iv (&context_cbc, 1, key, sizeof(key), ivl, ivr,
			     store, &ptr);
      for ( j=0 ; j<(int)sizeof(data) ; j++ )
	Blowfish_cbc_feed (&context_cbc, data[j]);
      Blowfish_cbc_stop (&context_cbc);
      if ( ptr-cipher != sizeof(expect_cbc) )
	return 0;
      for ( j=0 ; j<(int)sizeof(expect_cbc) ; j++ )
	if ( cipher[j] != expect_cbc[j] )
	  return 0;
      ptr = decipher;
      Blowfish_cbc_start_iv (&context_cbc, 0, key, sizeof(key), ivl, ivr,
			     store, &ptr);
      for ( j=0 ; j<(int)sizeof(expect_cbc) ; j++ )
	Blowfish_cbc_feed (&context_cbc, expect_cbc[j]);
      Blowfish_cbc_stop (&context_cbc);
#if 0 /* CBC mode does not preserve data size */
      if ( ptr-decipher != sizeof(data) )
	return 0;
#endif
      for ( j=0 ; j<(int)sizeof(data) ; j++ )
	if ( decipher[j] != data[j] )
	  return 0;
    }
  else
    {
      Blowfish_cfb_start_iv (&context_cfb, 1, key, sizeof(key), ivl, ivr,
			     store, &ptr);
      for ( j=0 ; j<(int)sizeof(data) ; j++ )
	Blowfish_cfb_feed (&context_cfb, data[j]);
      Blowfish_cfb_stop (&context_cfb);
      if ( ptr-cipher != sizeof(expect_cfb) )
	return 0;
      for ( j=0 ; j<(int)sizeof(expect_cfb) ; j++ )
	if ( cipher[j] != expect_cfb[j] )
	  return 0;
      ptr = decipher;
      Blowfish_cfb_start_iv (&context_cfb, 0, key, sizeof(key), ivl, ivr,
			     store, &ptr);
      for ( j=0 ; j<(int)sizeof(expect_cfb) ; j++ )
	Blowfish_cfb_feed (&context_cfb, expect_cfb[j]);
      Blowfish_cfb_stop (&context_cfb);
      if ( ptr-decipher != sizeof(data) )
	return 0;
      for ( j=0 ; j<(int)sizeof(data) ; j++ )
	if ( decipher[j] != data[j] )
	  return 0;
    }
  return 1;
}

void
do_test_2 (char cfb)
{
  if ( test_2 (cfb) )
    printf ("Test2 passed successfully for %s mode.\n",
	    cfb?"CFB":"CBC");
  else
    {
      printf ("Test2 FAILED for %s mode!\n",
	      cfb?"CFB":"CBC");
      all_ok = 0;
    }
}

int
main (void)
{
  all_ok = 1;

  do_test_0 (1,
	     0x00000000, 0x00000000, 0x00000000, 0x00000000,
	     0x4ef99745, 0x6198dd78);
  do_test_0 (2,
	     0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
	     0x51866fd5, 0xb85ecb8a);
  do_test_0 (3,
	     0x30000000, 0x00000000, 0x10000000, 0x00000001,
	     0x7d856f9a, 0x613063f2);
  do_test_0 (4,
	     0x11111111, 0x11111111, 0x11111111, 0x11111111,
	     0x2466dd87, 0x8b963c9d);
  do_test_0 (5,
	     0x01234567, 0x89abcdef, 0x11111111, 0x11111111,
	     0x61f9c380, 0x2281b096);
  do_test_0 (6,
	     0x11111111, 0x11111111, 0x01234567, 0x89abcdef,
	     0x7d0cc630, 0xafda1ec7);
  do_test_0 (7,
	     0x00000000, 0x00000000, 0x00000000, 0x00000000,
	     0x4ef99745, 0x6198dd78);
  do_test_0 (8,
	     0xfedcba98, 0x76543210, 0x01234567, 0x89abcdef,
	     0x0aceab0f, 0xc6a0a28d);
  do_test_0 (9,
	     0x7ca11045, 0x4a1a6e57, 0x01a1d6d0, 0x39776742,
	     0x59c68245, 0xeb05282b);
  do_test_0 (10,
	     0x0131d961, 0x9dc1376e, 0x5cd54ca8, 0x3def57da,
	     0xb1b8cc0b, 0x250f09a0);
  do_test_0 (11,
	     0x07a1133e, 0x4a0b2686, 0x0248d438, 0x06f67172,
	     0x1730e577, 0x8bea1da4);
  do_test_0 (12,
	     0x3849674c, 0x2602319e, 0x51454b58, 0x2ddf440a,
	     0xa25e7856, 0xcf2651eb);
  do_test_0 (13,
	     0x04b915ba, 0x43feb5b6, 0x42fd4430, 0x59577fa2,
	     0x353882b1, 0x09ce8f1a);
  do_test_0 (14,
	     0x0113b970, 0xfd34f2ce, 0x059b5e08, 0x51cf143a,
	     0x48f4d088, 0x4c379918);
  do_test_0 (15,
	     0x0170f175, 0x468fb5e6, 0x0756d8e0, 0x774761d2,
	     0x432193b7, 0x8951fc98);
  do_test_0 (16,
	     0x43297fad, 0x38e373fe, 0x762514b8, 0x29bf486a,
	     0x13f04154, 0xd69d1ae5);
  do_test_0 (17,
	     0x07a71370, 0x45da2a16, 0x3bdd1190, 0x49372802,
	     0x2eedda93, 0xffd39c79);
  do_test_0 (18,
	     0x04689104, 0xc2fd3b2f, 0x26955f68, 0x35af609a,
	     0xd887e039, 0x3c2da6e3);
  do_test_0 (19,
	     0x37d06bb5, 0x16cb7546, 0x164d5e40, 0x4f275232,
	     0x5f99d04f, 0x5b163969);
  do_test_0 (20,
	     0x1f08260d, 0x1ac2465e, 0x6b056e18, 0x759f5cca,
	     0x4a057a3b, 0x24d3977b);
  do_test_0 (21,
	     0x58402364, 0x1aba6176, 0x004bd6ef, 0x09176062,
	     0x452031c1, 0xe4fada8e);
  do_test_0 (22,
	     0x02581616, 0x4629b007, 0x480d3900, 0x6ee762f2,
	     0x7555ae39, 0xf59b87bd);
  do_test_0 (23,
	     0x49793ebc, 0x79b3258f, 0x437540c8, 0x698f3cfa,
	     0x53c55f9c, 0xb49fc019);
  do_test_0 (24,
	     0x4fb05e15, 0x15ab73a7, 0x072d43a0, 0x77075292,
	     0x7a8e7bfa, 0x937e89a3);
  do_test_0 (25,
	     0x49e95d6d, 0x4ca229bf, 0x02fe5577, 0x8117f12a,
	     0xcf9c5d7a, 0x4986adb5);
  do_test_0 (26,
	     0x018310dc, 0x409b26d6, 0x1d9d5c50, 0x18f728c2,
	     0xd1abb290, 0x658bc778);
  do_test_0 (27,
	     0x1c587f1c, 0x13924fef, 0x30553228, 0x6d6f295a,
	     0x55cb3774, 0xd13ef201);
  do_test_0 (28,
	     0x01010101, 0x01010101, 0x01234567, 0x89abcdef,
	     0xfa34ec48, 0x47b268b2);
  do_test_0 (29,
	     0x1f1f1f1f, 0x0e0e0e0e, 0x01234567, 0x89abcdef,
	     0xa7907951, 0x08ea3cae);
  do_test_0 (30,
	     0xe0fee0fe, 0xf1fef1fe, 0x01234567, 0x89abcdef,
	     0xc39e072d, 0x9fac631d);
  do_test_0 (31,
	     0x00000000, 0x00000000, 0xffffffff, 0xffffffff,
	     0x014933e0, 0xcdaff6e4);
  do_test_0 (32,
	     0xffffffff, 0xffffffff, 0x00000000, 0x00000000,
	     0xf21e9a77, 0xb71c49bc);
  do_test_0 (33,
	     0x01234567, 0x89abcdef, 0x00000000, 0x00000000,
	     0x24594688, 0x5754369a);
  do_test_0 (34,
	     0xfedcba98, 0x76543210, 0xffffffff, 0xffffffff,
	     0x6b5c5a9c, 0x5d9e0a5a);

  do_test_1 ( 1, 0xf9ad597c, 0x49db005e);
  do_test_1 ( 2, 0xe91d21c1, 0xd961a6d6);
  do_test_1 ( 3, 0xe9c2b70a, 0x1bc65cf3);
  do_test_1 ( 4, 0xbe1e6394, 0x08640f05);
  do_test_1 ( 5, 0xb39e4448, 0x1bdb1e6e);
  do_test_1 ( 6, 0x9457aa83, 0xb1928c0d);
  do_test_1 ( 7, 0x8bb77032, 0xf960629d);
  do_test_1 ( 8, 0xe87a244e, 0x2cc85e82);
  do_test_1 ( 9, 0x15750e7a, 0x4f4ec577);
  do_test_1 (10, 0x122ba70b, 0x3ab64ae0);
  do_test_1 (11, 0x3a833c9a, 0xffc537f6);
  do_test_1 (12, 0x9409da87, 0xa90f6bf2);
  do_test_1 (13, 0x884f8062, 0x5060b8b4);
  do_test_1 (14, 0x1f85031c, 0x19e11968);
  do_test_1 (15, 0x79d9373a, 0x714ca34f);
  do_test_1 (16, 0x93142887, 0xee3be15c);
  do_test_1 (17, 0x03429e83, 0x8ce2d14b);
  do_test_1 (18, 0xa4299e27, 0x469ff67b);
  do_test_1 (19, 0xafd5aed1, 0xc1bc96a8);
  do_test_1 (20, 0x10851c0e, 0x3858da9f);
  do_test_1 (21, 0xe6f51ed7, 0x9b9db21f);
  do_test_1 (22, 0x64a6e14a, 0xfd36b46f);
  do_test_1 (23, 0x80c7d7d4, 0x5a5479ad);
  do_test_1 (24, 0x05044b62, 0xfa52d080);

  do_test_2 (0);
  do_test_2 (1);

  return all_ok ? 0 : 1;
}
