#include "openpgp.h"
#include "sha1.h"
#include "twofish.h"

#include <stdio.h>
#include <string.h>

const char message[] =
"This is the secret message.\nThe answer to Life, the Universe, "
"and Everything, is: 42\n";

const char passphrase[] = "passphrase";

const char fname[] = "_CONSOLE";

static long armor_len = 0;
static long pgp_len = 0;
static long msg_len = 0;

#if defined(__GNUC__)
void
putout (char ch, void *junk __attribute__((__unused__)))
#else
void
putout (char ch, void *junk)
#endif
{
  putchar (ch);
  armor_len++;
}

void
putarmor (unsigned char byte, void *armor)
{
  ASCII_Armor_feed ((struct ascii_armor_handle *)armor, byte);
  pgp_len++;
}

void
putpgp (unsigned char byte, void *pgp)
{
  OpenPGP_feed ((struct openpgp_handle *)pgp, byte);
}

void
putcipher (unsigned char byte, void *ctx)
{
  Twofish_cfb_feed ((struct twf_cfb_ctx *)ctx, byte);
  msg_len++;
}

int
main (void)
{
  struct ascii_armor_handle armor;
  struct openpgp_handle pgp;
  unsigned char salt[8];
  unsigned char s2k[10];
  struct sha1_ctx sha1_ctx;
  unsigned char digest[20];
  unsigned char seskey[32];
  struct twf_cfb_ctx twf_ctx;
  unsigned char psiv[16];
  struct openpgp_handle msg;
  int i;

  ASCII_Armor_start (&armor, "PGP MESSAGE", NULL, putout, NULL);
  OpenPGP_start (&pgp, putarmor, &armor);
  for ( i=0 ; i<8 ; i++ )
    salt[i] = (29-i)*7;
  s2k[0] = 0x1;
  s2k[1] = OPENPGP_HASH_SHA1;
  memcpy (s2k+2, salt, 8);
  OpenPGP_symenc_seskey (&pgp, OPENPGP_CIPHER_TWOFISH256,
			 s2k, sizeof(s2k), 0);
  OpenPGP_end_packet (&pgp);
  OpenPGP_symenc_data (&pgp,
#if 1
		       (18 + predict_literal_data_len (strlen (fname),
						       strlen (message)))
#else
		       -1
#endif
		       );
  SHA1_start (&sha1_ctx);
  for ( i=0 ; i<8 ; i++ )
    SHA1_feed (&sha1_ctx, salt[i]);
  for ( i=0 ; passphrase[i] ; i++ )
    SHA1_feed (&sha1_ctx, passphrase[i]);
  SHA1_stop (&sha1_ctx, digest);
  memcpy (seskey, digest, 20);
  SHA1_start (&sha1_ctx);
  SHA1_feed (&sha1_ctx, 0);
  for ( i=0 ; i<8 ; i++ )
    SHA1_feed (&sha1_ctx, salt[i]);
  for ( i=0 ; passphrase[i] ; i++ )
    SHA1_feed (&sha1_ctx, passphrase[i]);
  SHA1_stop (&sha1_ctx, digest);
  memcpy (seskey+20, digest, 12);
  Twofish_cfb_start (&twf_ctx, 1, seskey, sizeof(seskey),
		     putpgp, &pgp);
  for ( i=0 ; i<16 ; i++ )
    psiv[i] = (i+7)*11;
  for ( i=0 ; i<16 ; i++ )
    Twofish_cfb_feed (&twf_ctx, psiv[i]);
  Twofish_cfb_feed (&twf_ctx, psiv[14]);
  Twofish_cfb_feed (&twf_ctx, psiv[15]);
  Twofish_cfb_resync (&twf_ctx);
  OpenPGP_start (&msg, putcipher, &twf_ctx);
  OpenPGP_literal_data (&msg, 'b', "_CONSOLE", strlen("_CONSOLE"), 0,
#if 1
			(strlen(message))
#else
			-1
#endif
			);
  for ( i=0 ; message[i] ; i++ )
    OpenPGP_feed (&msg, message[i]);
  OpenPGP_end_packet (&msg);
  OpenPGP_stop (&msg);
  Twofish_cfb_stop (&twf_ctx);
  OpenPGP_end_packet (&pgp);
  OpenPGP_stop (&pgp);
  ASCII_Armor_stop (&armor);
  printf ("Check: msg_len=%ld (predicted: %ld)\n",
	  msg_len,
	  predict_literal_data_len (strlen (fname),
				    strlen (message)));
  printf ("Check: pgp_len=%ld (predicted: %ld)\n",
	  pgp_len,
	  predict_symenc_seskey_len (sizeof(s2k), 0)
	  + predict_symenc_data_len (18 + msg_len));
  printf ("Check: armor_len=%ld (predicted: %ld)\n",
	  armor_len,
	  predict_armor_len ("PGP MESSAGE", NULL, pgp_len));
  return 0;
}
